<?php

class Andy_NewUserConversation_DataWriter_User extends XFCP_Andy_NewUserConversation_DataWriter_User
{
	protected function _postSave()
	{
		//########################################
		// send Conversation
		//########################################	
		
		// get parent
		$parent = parent::_postSave();	
	
		// declare variable
		$sendConversation = false;
		
		//########################################
		// When a new user registers, User.php runs
		// before Register.php. A new user account
		// is created and the "if" command below
		// will run only once because the
		// user_id is only change during new account
		// creation. We set the sendConversation
		// variable true and if there is no moderation
		// we will send the conversation immediatly.
		//########################################

		// new user just created, this only happens once
		if ($this->isChanged('user_id'))
		{
			$sendConversation = true;
		}
		
		//########################################
		// The "if" statement below runs anytime
		// the user account is updated. The user
		// account is updated when the account is 
		// first created, the admin changes the
		// account from moderated to valid, or the
		// user clicks the email verification link.	
		//########################################
		
		// continue only if changed user_sate and is valid
		if ($this->isChanged('user_state') && $this->get('user_state') == 'valid')
		{					
			// get options from Admin CP -> Options -> NewUserConversation -> User ID
			$userId = XenForo_Application::get('options')->newUserConversationUserId;
			
			// get options from Admin CP -> Options -> NewUserConversation -> Title
			$title = XenForo_Application::get('options')->newUserConversationTitle;
			
			// get options from Admin CP -> Options -> NewUserConversation -> Message
			$message = XenForo_Application::get('options')->newUserConversationMessage;
			
			// get options from Admin CP -> Options -> NewUserConversation -> Lock
			$lock = XenForo_Application::get('options')->newUserConversationLock;
			
			// get options from Admin CP -> Options -> NewUserConversation -> Leave Conversation
			$leaveConversation = XenForo_Application::get('options')->newUserConversationLeaveConversation;
					
			// set lock value
			if ($lock)
			{
				$lock = 0;
			}
			else
			{
				$lock = 1;
			}
			
			// set leave value
			if ($leaveConversation)
			{
				$leaveConversation = 'deleted';
			}
			else
			{
				$leaveConversation = 'active';
			}		
			
			// get database
			$db = XenForo_Application::get('db');		
			
			// run query (sender)
			$sender = $db->fetchRow("
				SELECT user_id, username
				FROM xf_user
				WHERE user_id = ?
			",$userId);			
			
			//########################################
			// get new registrations
			//########################################
			
			// moderation disabled - send conversation
			if ($sendConversation)
			{
				// run query
				$results = $db->fetchAll("
					SELECT user_id, username
					FROM xf_user
					WHERE user_id = " . $this->get('user_id') . "
				");
			}
			
			//########################################
			// Used when moderation is enabled.
			// After the User.php file runs,
			// the Register.php file runs. At this
			// time we insert into xf_new_user_conversation
			// a new row so we know a conversation has
			// not been sent yet. When the account 
			// becomes valid, we run the "if" below
			// so the conversation will be sent just once.	
			//########################################
			
			// moderation enabled - just made valid
			if (!$sendConversation)
			{
				// run query
				$results = $db->fetchAll("
					SELECT xf_user.user_id, xf_user.username
					FROM xf_user
					INNER JOIN xf_new_user_conversation ON xf_new_user_conversation.user_id = xf_user.user_id
					WHERE xf_user.user_state = 'valid'
				");
			}
			
			//########################################
			// send conversations
			//########################################		
			
			// loop
			foreach ($results as $k => $v)
			{
				// unset recipientArray
				unset($recipientArray);
				
				// create recipient array			
				$recipientArray[] = $v['user_id'];
				
				// replace {username} in message
				$message = str_replace('{username}', $v['username'], $message);			
		
				$conversationDw = XenForo_DataWriter::create('XenForo_DataWriter_ConversationMaster');
				$conversationDw->set('user_id', $sender['user_id']);
				$conversationDw->set('username', $sender['username']);
				$conversationDw->set('title', $title);
				$conversationDw->set('open_invite', 0);
				$conversationDw->set('conversation_open', $lock);
				$conversationDw->addRecipientUserIds($recipientArray);
				$messageDw = $conversationDw->getFirstMessageDw();
				$messageDw->set('message', $message);
				$messageDw->setOption(XenForo_DataWriter_ConversationMessage::OPTION_SET_IP_ADDRESS, false);
		
				$conversationDw->preSave();
				$conversationDw->save();
				
				//########################################
				// remove unread information for sender
				//########################################
				
				$db->query('
					UPDATE xf_user SET
						conversations_unread = IF(conversations_unread > 1, conversations_unread - 1, 0)
					WHERE user_id = ?
				', $userId);
	
				// get conversation data that was just created
				$conversation = $conversationDw->getMergedData();
				
				// get conversationId that was just created
				$conversationId = $conversation['conversation_id'];
				
				// get time
				$time = time();
	
				// update xf_conversation_recipient					
				$db->query('
					UPDATE xf_conversation_recipient SET
						last_read_date = "' . $time . '",
						recipient_state = "' . $leaveConversation . '"
					WHERE user_id = ' . $userId . '
					AND conversation_id = ' . $conversationId . '
				');
				
				// leaveConversation == active
				if ($leaveConversation == 'active')
				{
					// update xf_conversation_user					
					$db->query('
						UPDATE xf_conversation_user SET
							is_unread = 0
						WHERE owner_user_id = ' . $userId . '
						AND conversation_id = ' . $conversationId . '
					');	
				}
				
				// leaveConversation == deleted
				if ($leaveConversation == 'deleted')
				{
					// delete row
					$db->query("
						DELETE FROM xf_conversation_user
						WHERE conversation_id = " . $conversationId . "
						AND owner_user_id = ?
					",$userId);
				}
				
				//########################################
				// delete row in xf_new_user_conversation
				//########################################
				
				// delete row
				$db->query("
					DELETE FROM xf_new_user_conversation
					WHERE user_id = " . $v['user_id'] . "
				");		
			}
		}
		
		// return parent
		return $parent;
	}
}